@@ -4,26 +4,144 @@
 # Copyright (C) 2011 Loic Jaquemet loic.jaquemet+python@gmail.com
 #
 
+'''
+This module is the main aspect of haystack.
+The whole point of this module is to search a memory space for a valid 
+C structure.
+'Valid' means:
+  * a pointer should have a pointer value.
+  * a string whould be a string.
+  * a constrainged integer should be in a range of acceptable values.
+  etc...
+
+You can implement your own structures easily by registering your module 
+containing ctypes Structure :
+
+  # mymodule.py
+  from haystack import model
+  
+  [... structure definition .. ]
+  
+  model.registerModule(sys.modules[__name__])
+
+--------------------------------------------------------------------------------
+
+Structure definition is done strictly within ctypes habits.
+But as a bonus, you can add some constraints on the structure members by adding
+a member expectedValues on the Python object.
+
+Example :
+
+  # ctypes_openssl.py
+  from haystack import model
+
+  class RSA(OpenSSLStruct):
+    """ rsa/rsa.h """
+    _fields_ = [
+    ("pad",  ctypes.c_int), 
+    ("version",  ctypes.c_long),
+    ("meth",ctypes.POINTER(BIGNUM)),#const RSA_METHOD *meth;
+    ("engine",ctypes.POINTER(ENGINE)),#ENGINE *engine;
+    ('n', ctypes.POINTER(BIGNUM) ), ## still in ssh memap
+    ('e', ctypes.POINTER(BIGNUM) ), ## still in ssh memap
+    ('d', ctypes.POINTER(BIGNUM) ), ## still in ssh memap
+    ('p', ctypes.POINTER(BIGNUM) ), ## still in ssh memap
+    ('q', ctypes.POINTER(BIGNUM) ), ## still in ssh memap
+    ('dmp1', ctypes.POINTER(BIGNUM) ),
+    ('dmq1', ctypes.POINTER(BIGNUM) ),
+    ('iqmp', ctypes.POINTER(BIGNUM) ),
+    ("ex_data", CRYPTO_EX_DATA ),
+    ("references", ctypes.c_int),
+    ("flags", ctypes.c_int),
+    ("_method_mod_n", ctypes.POINTER(BN_MONT_CTX) ),
+    ("_method_mod_p", ctypes.POINTER(BN_MONT_CTX) ),
+    ("_method_mod_q", ctypes.POINTER(BN_MONT_CTX) ),
+    ("bignum_data",ctypes.POINTER(ctypes.c_ubyte)), ## moue c_char_p ou POINTER(c_char) ?
+    ("blinding",ctypes.POINTER(BIGNUM)),#BN_BLINDING *blinding;
+    ("mt_blinding",ctypes.POINTER(BIGNUM))#BN_BLINDING *mt_blinding;
+    ]
+    expectedValues={
+      "pad": [0], 
+      "version": [0], 
+      "references": RangeValue(0,0xfff),
+      "n": [NotNull],
+      "e": [NotNull],
+      "d": [NotNull],
+      "p": [NotNull],
+      "q": [NotNull],
+      "dmp1": [NotNull],
+      "dmq1": [NotNull],
+      "iqmp": [NotNull]
+    }
+    def loadMembers(self, mappings, maxDepth):
+      print 'example'
+      if not LoadableMembers.loadMembers(self, mappings, maxDepth):
+        log.debug('RSA not loaded')
+        return False
+      return True
+
+  # register to haystack
+  model.registerModule(sys.modules[__name__])
+
+--------------------------------------------------------------------------------
+
+As an added bonus, you can also use ctypeslib code generator to autogenerate 
+ctypes python structure from C headers.
+You should have a look into that : h2py && xml2py 
+Anyway, haystack will support you with your generated headers.
+Put the generated headers in a separate file, and put your expected values in an
+other file. You can then register the autogenerated module.
+
+Example :
+
+  # generated headers are in ctypes_putty_generated.py
+  # ctypes_putty.py
+  import ctypes_putty_generated as gen
+
+  ################ START copy generated classes ##########################
+  # copy generated classes (gen.*) to this module as wrapper
+  model.copyGeneratedClasses(gen, sys.modules[__name__])
+
+  # register all classes (gen.*, locally defines, and local duplicates) to haystack
+  # create plain old python object from ctypes.Structure's, to picke them
+  model.registerModule(sys.modules[__name__])
+
+  RSAKey.expectedValues={
+      'bits': [NotNull],
+      'bytes': [NotNull],
+      'modulus': [NotNull],
+      'exponent': [NotNull],
+      'private_exponent': [NotNull],
+      'p': [NotNull],
+      'q': [NotNull],
+      'iqmp': [NotNull]
+    }
+  [... loads of constraints on putty structures... ]
+
+--------------------------------------------------------------------------------
+
+Enjoy.
+
+'''
+
 __author__ = "Loic Jaquemet loic.jaquemet+python@gmail.com"
 
-import ctypes,os, types
-from struct import pack,unpack
-from memory_mapping import readProcessMappings
+import ctypes
 import logging
-log=logging.getLogger('model')
+from utils import *
 
-#import copy
-''' replace c_char_p '''
+log = logging.getLogger('model')
+
+# replace c_char_p so we can have our own CString 
 if ctypes.c_char_p.__name__ == 'c_char_p':
   ctypes.original_c_char_p = ctypes.c_char_p
 
-''' keep orig class '''
+# keep orig class and Use our model instead as base Structure class
 if ctypes.Structure.__name__ == 'Structure':
   ctypes.original_Structure = ctypes.Structure
 
-#__refs = list()
-#__register = dict()
-
+# The book registers all haystack modules, and classes, and can keep 
+# some pointer refs on memory allocated within special cases...
 class _book(object):
   modules = set()
   classes = dict()
@@ -85,194 +203,15 @@ def register(klass):
 def registeredModules():
   return sys.modules[__name__].__book.getModules()
 
-''' returns if the address of the struct is in the mapping area
-'''
-def is_valid_address(obj,mappings, structType=None):
-  '''static int is_valid_address(unsigned long addr, mappings_t *mappings) {'''
-  # check for null pointers
-  #print 'is_valid_address'
-  addr=getaddress(obj)
-  if addr == 0:
-    return False
-  return is_valid_address_value(addr,mappings,structType)
-
-def is_valid_address_value(addr,mappings,structType=None):
-  for m in mappings:
-    if addr in m:
-      # check if end of struct is ALSO in m
-      if (structType is not None):
-        s=ctypes.sizeof(structType)
-        if (addr+s) not in m:
-          return False
-      return m
-  return False
-
-def is_address_local(obj, structType=None):
-  ''' costly , checks if obj is mapped to local memory '''
-  addr=getaddress(obj)
-  if addr == 0:
-    return False
-  class P:
-    pid=os.getpid()
-  mappings= readProcessMappings(P())
-  return is_valid_address(obj,mappings, structType)
-
-''' returns the address of the struct
-'''
-def getaddress(obj):
-  # check for null pointers
-  #print 'getaddress'
-  if bool(obj):
-    if not hasattr(obj,'contents'):
-      return 0
-    #print 'get adressses is '
-    return ctypes.addressof(obj.contents)
-  else:
-    #print 'object pointer is null'
-    return 0  
-
-def container_of(memberaddr, typ, membername):
-  '''      const typeof( ((typ *)0)->member ) *__mptr = (ptr);    
-        (type *)( (char *)__mptr - offsetof(type,member) );}) '''
-  return typ.from_address( memberaddr - offsetof(typ, membername) )
-
-def offsetof(typ, membername):
-  T=typ()
-  return ctypes.addressof(  getattr(T,membername) ) - ctypes.addressof(T)
-
-
-''' MISSING
-d 	double 	float 	8 	(4)
-p 	char[] 	string 	  	 
-'''
-bytestr_fmt={
-  'c_bool': '?',
-  'c_char': 'c',
-  'c_byte': 'b',
-  'c_ubyte': 'B',
-  'c_short': 'h',
-  'c_ushort': 'H',
-  'c_int': 'i', #c_int is c_long
-  'c_uint': 'I',
-  'int': 'i', 
-  'c_long': 'l', #c_int is c_long
-  'c_ulong': 'L',
-  'long': 'q', 
-  'c_longlong': 'q',
-  'c_ulonglong': 'Q',
-  'c_float': 'f', ## and double float ?
-  'c_char_p': 's',
-  'c_void_p': 'P'
-  }
-def array2bytes_(array, typ):
-  arrayLen = len(array)
-  if arrayLen == 0:
-    return b''
-  if typ not in bytestr_fmt:
-    log.warning('Unknown ctypes to pack: %s'%(typ))
-    return None
-  fmt=bytestr_fmt[typ]
-  sb=b''
-  for el in array:
-    sb+=pack(fmt, el)
-  return sb
-
-def array2bytes(array):
-  if not isBasicTypeArrayType(array):
-    return b'NOT-AN-BasicType-ARRAY'
-  # BEURK
-  typ='_'.join(type(array).__name__.split('_')[:2])
-  return array2bytes_(array,typ)
-
-def bytes2array(bytes, typ):
-  typLen=ctypes.sizeof(typ)
-  if len(bytes)%typLen != 0:
-    raise ValueError('thoses bytes are not an array of %s'%(typ))
-  arrayLen=len(bytes)/typLen
-  array=(typ*arrayLen)()
-  if arrayLen == 0:
-    return array
-  if typ.__name__ not in bytestr_fmt:
-    log.warning('Unknown ctypes to pack: %s'%(typ))
-    return None
-  fmt=bytestr_fmt[typ.__name__]
-  sb=b''
-  for i in range(0,arrayLen):
-    array[i]=unpack(fmt, bytes[typLen*i:typLen*(i+1)])[0]
-  return array
-
-
-def pointer2bytes(attr,nbElement):
-  # attr is a pointer and we want to read elementSize of type(attr.contents))
-  if not is_address_local(attr):
-    return 'POINTER NOT LOCAL'
-  firstElementAddr=getaddress(attr)
-  array=(type(attr.contents)*nbElement).from_address(firstElementAddr)
-  # we have an array type starting at attr.contents[0]
-  return array2bytes(array)
-
-def isCTypes(obj):
-  ''' pointer or type'''
-  return  (type(obj).__module__ in ['ctypes','_ctypes']) 
-    
-def isBasicType(obj):
-  return  (type(obj).__module__ in ['ctypes','_ctypes','__builtin__']) 
-
-def isStructType(obj):
-  ''' a struct is what WE have created '''
-  #return isinstance(obj,LoadableMembers)
-  return isinstance(obj, ctypes.Structure)
-  # or use obj.classRef
-  
-def isPointerType(obj):
-  if isBasicType(obj) or isStructType(obj):
-    return False
-  return 'PointerType' in type(obj).__class__.__name__ 
-
-def isBasicTypeArrayType(obj):
-  if isArrayType(obj):
-    if len(obj) == 0:
-      return True
-    if isBasicType(obj[0]):
-      return True
-  return False
-
-def isArrayType(obj):
-  return 'ArrayType' in type(obj).__class__.__name__
-
-def isFunctionType(obj):
-  return 'CFuncPtrType' in type(obj).__class__.__name__
-
-def isCStringPointer(obj):
-  return obj.__class__.__name__ == 'CString'
-
-def isUnionType(obj):
-  return isinstance(obj,ctypes.Union) and not isCStringPointer(obj)
-
-
-class IgnoreMember:
-  def __contains__(self,obj):
-    return True
-
-class RangeValue:
-  def __init__(self,low,high):
-    self.low=low
-    self.high=high
-  def __contains__(self,obj):
-    return self.low <= obj <= self.high
-  def __eq__(self,obj):
-    return self.low <= obj <= self.high
-
-class NotNullComparable:
-  def __contains__(self,obj):
-    return bool(obj)
-  def __eq__(self,obj):
-    return bool(obj)
-
-NotNull=NotNullComparable()
 
 class CString(ctypes.Union):
-  ''' ctypes.c_char_p can not be used for memory parsing. it tries to load the string itself '''
+  ''' 
+  This is our own implementation of a string for ctypes.
+  ctypes.c_char_p can not be used for memory parsing, as it tries to load 
+  the string itself without checking for pointer validation.
+  
+  it's basically a Union of a string and a pointer.
+  '''
   _fields_=[
   ("string", ctypes.original_c_char_p),
   ("ptr", ctypes.POINTER(ctypes.c_ubyte) )
@@ -284,29 +223,21 @@ class CString(ctypes.Union):
   pass
 
 
-
-#debug
-def printWhois(attr):
-  print ' : isBasicType(attr): %s bool(attr): %s'%(isBasicType(attr) ,bool(attr)) 
-  print ' : isCStringPointer(attr): %s isStructType(attr): %s'%(isCStringPointer(attr) ,isStructType(attr)) 
-  print ' : isArrayType(attr): %s isBasicTypeArrayType(attr): %s'%(isArrayType(attr) ,isBasicTypeArrayType(attr)) 
-  print ' : isPointerType(attr): %s type(attr) %s '%(isPointerType(attr),type(attr) ) 
-  print ' : ',attr.__class__.__name__, type(attr)
-
-
 class LoadableMembers(ctypes.Structure):
-  ''' ctypes.POINTER types for automatic address space checks '''
+  ''' 
+  This is the main class, to be inherited by all ctypes structure.
+  It adds a generic validaiton framework, based on simple assertion, 
+  and on more complex constraint on members values.
+    
+  '''
   classRef=dict()
   expectedValues=dict()
 
   def isValid(self,mappings):
-    '''  checks if each members has coherent data  '''
-    valid = self._isValid(mappings)
-    log.debug('%s isValid = %s'%(self.__class__.__name__,valid))
-    return valid
+    ''' 
+    Checks if each members has coherent data 
 
-  def _isValid(self,mappings):
-    ''' For each Field, check on of the three case, 
+    For each Field, check on of the three case, 
       a) basic types (check for expectedValues), 
         if field as some expected values in expectedValues
            check field value against expectedValues[fieldname]
@@ -326,6 +257,12 @@ class LoadableMembers(ctypes.Structure):
            check getaddress against is_valid_address() 
               if False, return False, else continue
     '''
+    valid = self._isValid(mappings)
+    log.debug('%s isValid = %s'%(self.__class__.__name__,valid))
+    return valid
+
+  def _isValid(self,mappings):
+    ''' real implementation.  '''
     # precheck for quick unvalidation
     # myfields=dict(self._fields_)  ## some fields are initialised...
     _fieldsTuple = [ (f[0],f[1]) for f in self._fields_] 
@@ -355,10 +292,7 @@ class LoadableMembers(ctypes.Structure):
     return True
     
   def _isValidAttr(self,attr,attrname,attrtype,mappings):
-    # check this attr
-    if attrname in [] :
-      print 'Ivalid ',repr(self)
-      printWhois(attr)
+    ''' Validation of a single member '''
     # a) 
     if isBasicType(attr):
       if attrname in self.expectedValues:
@@ -437,12 +371,12 @@ class LoadableMembers(ctypes.Structure):
       #log.warning('Union are not validated , yet ')
       return True
     log.error('What type are You ?: %s/%s'%(attrname,attrtype))
-    printWhois(attr)
     return True
 
   def _isLoadableMember(self, attr):
     '''
-      Un VoidPointer ne doit pas etre Loadable
+      Check if the member is loadable.
+      A c_void_p cannot be load generically, You have to take care of that.
     '''
     attrtype=type(attr)
     return ( (isPointerType(attr) and ( attrtype in self.classRef) and bool(attr) ) or
@@ -490,15 +424,6 @@ class LoadableMembers(ctypes.Structure):
     return True
     
   def _loadMember(self,attr,attrname,attrtype,mappings, maxDepth):
-    ### debug
-    if attrname in []:
-      #if True:
-      print repr(self)
-      printWhois(attr)
-      print ' : _isLoadableMember() %s'%(self._isLoadableMember(attr) )
-      print ' ********** 0x%lx ' % ctypes.addressof(attr)
-      if bool(attr):
-        print ' ********** 0x%lx ' % ctypes.addressof(attr.contents)
     # skip static basic data members
     if not self._isLoadableMember(attr):
       log.debug("%s %s not loadable  bool(attr) = %s"%(attrname,attrtype, bool(attr)) )
@@ -565,8 +490,6 @@ class LoadableMembers(ctypes.Structure):
         return True
       log.debug("%s %s loading from 0x%lx (is_valid_address: %s)"%(attrname,attr,attr_obj_address, memoryMap ))
       ##### VALID INSTR.
-      if attrname == 'real_parent':
-        print 'real_parent',__book.refs
       attr.contents=_attrType.from_buffer_copy(memoryMap.readStruct(attr_obj_address, _attrType ))
       # save that ref and original addr so we dont need to recopy it later
       keepRef( attr.contents, _attrType,attr_obj_address)
@@ -584,7 +507,9 @@ class LoadableMembers(ctypes.Structure):
     return True
   
   def toString(self,prefix=''):
-    ''' return a string formatted description of this Structure. '''
+    ''' Returns a string formatted description of this Structure. 
+    The returned string should be python-compatible...
+    '''
     s="%s # %s\n"%(prefix,repr(self) )
     _fieldsTuple = [ (f[0],f[1]) for f in self._fields_] 
     for field,typ in _fieldsTuple:
@@ -661,12 +586,14 @@ class LoadableMembers(ctypes.Structure):
       elif type(attr) is long or type(attr) is int:
         s+='%s : %s\n'%(field, hex(attr) )  
       else:
-        #print '*** attr cannot be __str__ ***',field, type(attr)
         s+='%s : %s\n'%(field, repr(attr) )  
     return s
     
   def toPyObject(self):
-    ''' returns a Plain Old python object as a perfect copy of this ctypes object.
+    ''' 
+    Returns a Plain Old python object as a perfect copy of this ctypes object.
+    array would be lists, pointers, inner structures, and circular 
+    reference should be handled nicely.
     '''
     # get self class.
     #log.info("%s %s %s_py"%(self.__class__.__module__, sys.modules[self.__class__.__module__], self.__class__.__name__) )
@@ -723,7 +650,8 @@ class LoadableMembers(ctypes.Structure):
 
 
 class pyObj(object):
-  ''' Base class for a plain old python object.
+  ''' 
+  Base class for a plain old python object.
   all haystack/ctypes classes will be translated in this format before pickling.
   
   Operations :
@@ -754,7 +682,6 @@ class pyObj(object):
       s = '%s,'%( attr.toString(prefix,maxDepth) )
     else:
       s = '%s,'%(repr(attr) )
-      #print 'ELSE type: %s %s'%(type(attr), type(type(attr)) )
     return s
 
   def __len__(self):
@@ -865,10 +792,13 @@ def createPOPOClasses( targetmodule ):
   return
 
 def registerModule( targetmodule ):
-  ''' register a ctypes module. To be run by target module.
-      all members is module will be registered, against their pointer types,
-      in a lookup table
-      Creates POPO's to be able to unpickle ctypes.
+  ''' 
+  Registers a ctypes module. To be run by target module.
+  
+  All members in this module will be registered, against their pointer types,
+  in a lookup table.
+  
+  Creates POPO's to be able to unpickle ctypes.
   '''
   if targetmodule in registeredModules():
     log.warning('Module %s already registered. Skipping.'%(targetmodule))
@@ -890,16 +820,17 @@ def isRegistered(cls):
   return __book.isRegisteredType(cls)
 
 # create local POPO ( lodableMembers )
-createPOPOClasses(sys.modules[__name__] )
+#createPOPOClasses(sys.modules[__name__] )
+LoadableMembers_py = type('%s.%s_py'%(__name__, LoadableMembers),( pyObj ,),{})
 # register LoadableMembers 
 register(LoadableMembers)
 
 
-''' replace c_char_p - it can handle memory parsing without reading it '''
+# replace c_char_p - it can handle memory parsing without reading it 
 if ctypes.c_char_p.__name__ == 'c_char_p':
   ctypes.c_char_p = CString
 
-''' switch class - we need our methods on ctypes.Structures for generated classes to work  '''
+# switch class - we need our methods on ctypes.Structures for generated classes to work  
 if ctypes.Structure.__name__ == 'Structure':
   ctypes.Structure = LoadableMembers
 
